-module(server_fun).
-export([main/0]).

start_server() ->
    Pid = spawn(fun server_fun/0),
    catch unregister(server),
    register(server, Pid).

% Modified server. We now sends the function as part of the message.
server_fun() ->
    receive
        {compute, Fun, Args, From} ->
            % Note the spawn. This makes the server available for new requests faster.
            spawn(
              fun() ->
                      % This is one way of applying a function to a parameter list
                      Ans = apply(Fun, Args),
                      % Some sleep, just to show some stuff in a later module
                      timer:sleep(round(Ans) * 10),
                      From ! {answer, Ans}
              end)
    end,

    server_fun().


main() ->
    start_server(),

    % Note how we can create functions to send as an argument, and that we pass
    % the created function's arguments as a list ([10, 2]).
    % For addition, use fun(X, Y) -> X + Y end
    % For subtracting, use fun(X, Y) -> X - Y end
    server ! {compute, fun(X, Y) -> X * Y end, [10, 2], self()},
    receive
        Res ->
            io:fwrite("~p~n", [Res])
    end.
